/**
*   Hi, my name is Daniel Madaler
*   daniel (at) madaler (dot) de
*
*   The code is freely available at www.pseudopunkt.de, no license at all.
*
*   This code has the goal to act as an example and test for my abilities in Javascript.
*   It is a simple example using canvas, Javascript and a little bit of jQuery.
*   It was created quickly (one evening) and acts as a simple demo/prototype
*
*   It is normal Chess. Be aware that you currently can only move the pawns forward. You can't capture opponent's pieces.
*
*/


/*global canvas,pieces,currentTurn,selectedPiece,context,allowedMoves*/
function draw() {
    var canvas = document.getElementById('chess'); // get canvas
        
    /**
    * draws the board tiles
    */
    var drawBoard = function() {
        var rowCount = getRowCount();
        // iterate through each row
        for (var i = 0; i < rowCount; i++) {
            drawRow(i);
        }
    };

    /**
    * draw each row seperatly
    */
    var drawRow = function(rowCounter) {
        // iterate through each tile
        for (var i = 0; i < getRowCount(); i++) {
            drawTile(rowCounter, i);
        }
    };

    /**
    * actually draw the tiles
    */
    var drawTile = function(rowCounter, tileCounter) {
        var tileSize = getTileSize();
    
        // colors for the tiles
        var colorA = "#E0E0E0"; 
        var colorB = "#808080";
        
        var color;

        // decide which tile color the tile should be
        if (rowCounter % 2) {
            color = (tileCounter % 2 ? colorA:colorB);
        } else {
            color = (tileCounter % 2 ? colorB:colorA);
        }

        // actually draw the tiles
        context.fillStyle = color;
        var x = rowCounter * tileSize;
        var y = tileCounter * tileSize;

        context.fillRect(x, y, tileSize, tileSize);

    };

    /**
    * draw both teams
    */
    var drawPieces = function() {
        drawTeam(pieces.white, "white");
        drawTeam(pieces.black, "black");
    };
    
    /**
    * TODO create/find graphics to make this part easier and the whole thing prettier
    */
    var drawTeam = function(pieces, color) {
        var tileSize = getTileSize();
        var x,y,textWidth, metric;
        
        for (var i = 0; i < pieces.length; i++) {
            context.font="20px Arial";
            context.fillStyle = color;
            
            // find out the actual size of the text to put it in the center
            textWidth = context.measureText(pieces[i].piece).width;
            
            // find coords for text
            x = pieces[i].col * tileSize + tileSize/2 - Math.round(textWidth/2); // put text in horizontal center of cell
            y = pieces[i].row * tileSize + 38; // put text in vertical center of the cell

            // write the text onto the canvas
            context.fillText(pieces[i].piece,x,y);
        }
    };
    
    /**
    *
    */
    var getRowCount = function() {
        return 8;
    }
    
    /**
    *
    */
    var getTileSize = function() {
        var rowCount = getRowCount();
        return canvas.height / rowCount; // 8 rows
    }
    
    /**
    * returns tile of given coordinate, must be inside the canvas
    */
    var findRowAndColumnForCoordinate = function(x, y) {
        
        var tileSize = getTileSize();
        var rowCount = getRowCount();

        tile = {
            "row": Math.floor(y / tileSize),
            "col": Math.floor(x / tileSize)
        };
        
        if (tile.row < 0 || tile.col < 0 || tile.row > rowCount - 1 || tile.col > rowCount - 1) {
            return null;
        } else {
            return tile;
        }
    };
        
    /**
    *
    */
    var getPieceOnTile = function(tile, team) {
        var piece = null;
        for (var i = 0; i < team.length; i++) {
            if (team[i].row === tile.row && team[i].col === tile.col) {
                piece = team[i];
            }
        }
        
        return piece;
    };
    
    /**
    * select peace and highlight the tile and the possible move tiles
    */
    var selectPiece = function(piece) {
        if (selectedPiece !== null) {
            selectedPiece = null;
            console.log('ERROR');
            return;
        }
        highlightTile(piece, "orange");
        selectedPiece = piece;
        drawPossibleMove();
    };
    
    /**
    *
    */
    var highlightTile = function(piece, color) {
        var tileSize = getTileSize();
        context.lineWidth = "2";
        context.strokeStyle = color;
        context.strokeRect(piece.col * tileSize, piece.row * tileSize, tileSize, tileSize);
    };
    
    /**
    *
    */
    var drawPossibleMove = function() {
        var tiles = getAllowedMove();
    };
    
    /**
    *
    */
    var getAllowedMove = function() {
        
        switch (selectedPiece.piece) {
            case "pawn": allowedMoves = pawnAllowedMove();
                         break;
            case "rook": // TODO
                         break;
            case "knight": //TODO
                         break;
            case "bishop": //TODO
                         break;
            case "king": //TODO
                         break;
            case "queen": //TODO
                         break;

        }
        
    };
    
    /**
    *   Move pawn forward
    *   Cannot move forward on occupied tile
    *   TODO move on occupied piece diagonally
    */
    var pawnMove = function(piece, highlightedPieces) {
        piece.row = (currentTurn === 0 ? piece.row - 1:piece.row+1);

        if (getPieceOnTile(piece,pieces.black) !== null || getPieceOnTile(piece,pieces.white) !== null) {
            return highlightedPieces;
        }
        highlightedPieces[highlightedPieces.length] = $.extend(true,{},piece);
        highlightTile(piece, "blue");
        return highlightedPieces;
    }
    
    /**
    *
    */
    var pawnAllowedMove = function() {
        // TODO move on enemy piece

        var piece = $.extend(true,{},selectedPiece); // we need a clone, not a reference

        var highlightedPieces = [];
        highlightedPieces[highlightedPieces.length] = $.extend(true,{},piece);

        
        if (currentTurn === 0) {
             highlightedPieces = pawnMove(piece, highlightedPieces);
            
            if (piece.row == 5) {
                highlightedPieces = pawnMove(piece, highlightedPieces);
            }
        } else {
            highlightedPieces = pawnMove(piece, highlightedPieces);
            
            if (piece.row == 2) {
                highlightedPieces = pawnMove(piece, highlightedPieces);
            }
        }
        return highlightedPieces;
    };
    
    /**
    *
    */
    var movePiece = function(tile) {
        // This check is to see if the player clicked on the selected piece. This unlocks the selection and redraws the board.
        // otherwise move the piece
        var found = false;
        for (var i = 0; i < allowedMoves.length; i++) {
            if (allowedMoves[i].row === tile.row && allowedMoves[i].col === tile.col) {
                if (JSON.stringify(allowedMoves[i]) !== JSON.stringify(selectedPiece) ) {
                    movePieceToTile(tile);
                }
            }
            
        }

        // always redraw the whole board
        drawBoard();
        drawPieces();
        selectedPiece = null;
    };
    
    /**
    *
    */
    var movePieceToTile = function(tile) {
        var teamPieces = (currentTurn === 0 ? pieces.white:pieces.black);

        for (var i = 0; i < teamPieces.length; i++) {
            if (teamPieces[i].row === selectedPiece.row && teamPieces[i].col === selectedPiece.col) {
                // set new position, thanks to referenced objects
                teamPieces[i].row = tile.row;
                teamPieces[i].col = tile.col;
                currentTurn = (currentTurn === 0 ? 1:0);
            }
        }
       
        
    }
    
    
    // check if canvas is support by browser
    if (canvas.getContext) {
        var context = canvas.getContext('2d');
        var currentTurn = 0; // current turn 0 is white, 1 is black
        var selectedPiece = null;
        var allowedMoves = null;
        
        // get pieces object from json
        // use jquery for convenience
        $.getJSON('data/pieces.json', function(json) { 
            pieces = json;
            
            // draw board and pieces into canvas
            drawBoard();
            drawPieces();
                      
            
            
            // event listener to check click events on the board
            canvas.addEventListener('click', tileClick, false);
        });
        
    } else {
        // If canvas not supported, throw simple error message
        $('#board').prepend('<b>Your browser does not support Canvas. Please update your browser to something from this century!<br/>Internet Explorer 9+, Firefox, Opera, Chrome, and Safari support the canvas element.</b>');
    }
    
    /**
    * EventListener for clicks on the board
    */
    var tileClick = function(data) {
        var piece = null;
        var clickedTile = findRowAndColumnForCoordinate(data.clientX, data.clientY);
        if (clickedTile === null) {
            return;
        }
        var team = (currentTurn === 0 ? pieces.white:pieces.black);

        if (selectedPiece === null) {
            piece = getPieceOnTile(clickedTile, team);

            if (piece !== null) {
                selectPiece(piece);
            }
        } else {
            movePiece(clickedTile);
        }
        
        return;
    };
}

